package binarization;

import java.util.Scanner;

/**
 * Controlador da instanciacao via carregadores.
 *
 * @author denis
 * @see ILoader
 */
public class LoadManager {

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private LoadManager() {
	}

	/**
	 * Devolve uma instancia da classe dada a partir da linha dada. Esta deve ser
	 * da seguinte forma:<ol>
	 * 
	 * <li>Nome completo da classe carregadora <em>(e.g. org.tempuri.MyLoader)</em></li>
	 * <li>Os parametros de construcao da classe separados por <em>whitespace</em></li>
	 * 
	 * </ol>
	 * 
	 * @param <T> a classe da instancia devolvida
	 * @param input a linha de entrada com a classe carregadora e os parametros de construcao
	 * @param returnClass o objeto que representa a classe da instancia
	 * @return uma instancia da classe dada a partir da linha dada
	 * 
	 * @throws ClassNotFoundException caso a classe carregadora nao seja encontrada
	 * @throws InstantiationException caso a classe carregadora nao possua construtor sem parametros (obrigatorio) 
	 * @throws IllegalAccessException caso o construtor da classe carregadora nao seja publico
	 * @throws IllegalArgumentException caso a linha de entrada nao possua o nome da classe carregadora
	 * @throws ClassCastException caso a classe carregadora dada nao implemente {@link ILoader}
	 */
	public static <T> T load(String input, Class<T> returnClass) throws ClassNotFoundException, InstantiationException, IllegalAccessException {
		final Scanner scanner = new Scanner(input);

		if (!scanner.hasNext()) {
			throw new IllegalArgumentException("Missing the loader class name.");
		}

		final String className = scanner.next();
		final Class<? extends ILoader> classFor = Class.forName(className).asSubclass(ILoader.class);

		/*se nao for do tipo correto, deveria dar erro agora*/
		@SuppressWarnings("unchecked")
		final ILoader<T> loader = (ILoader<T>) classFor.newInstance();

		return returnClass.cast(loader.load(scanner));
	}
}
