package binarization;

import binarization.data.IBinaryImage;
import binarization.data.IGrayLevelImage;
import binarization.data.Images;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.Scanner;

/**
 * Ponto de entrada para utillizar os metodos de binarizacao e avalia-los ao mesmo tempo.
 * Produz as imagens binarizadas, a avaliacao de cada imagem e a media das avaliacoes,
 * alem do ranking obtido das avaliacoes.
 * 
 * <p>E verificado se o GT esqueleto sera utilizado na primeira linha da entrada,
 * assumindo que havera outros em cada outra linha do programa. Se nao houver GT
 * esqueleto ne primeira linha, somente as metricas que nao a usam serao utilizadas.
 * Caso ela esteja na primeira linha, mas nao haja GT esqueleto em algumas das
 * restantes, o valor das metricas que utilizam a GT esqueleto e indefinido.</p>
 *
 * @author denis
 */
public class Main {

	private static final String FILE_SEPARATOR = System.getProperty("file.separator");
	private static final String CURRENT_TIME = new SimpleDateFormat("dd.MM.yyyy_kk.mm.ss").format(Calendar.getInstance().getTime());
	private static final String RESULTS_DIRNAME = "bin_results_";
	private static final String AVERAGES_DIRNAME = "averages";

	/**
	 * Enum que define o tipo de ranking a ser usado. Deve ser substituido por
	 * polimorfismo caso o numero de tipos aumente.
	 */
	public enum RankingType {

		/** Ranking da media de todos resultados, como no DIBCO 2009 */
		AVERAGE("ranking_average.txt") {

			@Override
			Collection<Ranking> newRankings(Results results, Collection<IBinarizationMethod> methods, IBinaryImageMetric metric) {
				return Collections.singletonList(Ranking.newAverageRanking(results, methods, metric));
			}
		},
		/** Ranking de cada resultados, como no DIBCO 2011 */
		PER_IMAGE("ranking_per_image.txt") {

			@Override
			Collection<Ranking> newRankings(Results results, Collection<IBinarizationMethod> methods, IBinaryImageMetric metric) {
				return Ranking.newPerImageRankings(results, methods, metric);
			}
		};
		private final String filename;

		private RankingType(String filename) {
			this.filename = filename;
		}

		String filename() {
			return filename;
		}

		abstract Collection<Ranking> newRankings(Results results, Collection<IBinarizationMethod> methods, IBinaryImageMetric metric);
	}

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private Main() {
	}

	/**
	 * @param args args[0] possui o arquivo de entrada com as imagens;
	 *             args[1], com as classes carregadoras de metodos;
	 *             args[2], de metricas;
	 */
	public static void main(String[] args) {
		if (args.length < 3) {
			System.out.println("Requer um arquivo com as imagens de entrada.");
			System.exit(-1);
		}
		final String filename = args[0];

		System.out.println("Entrada: " + filename);
		System.out.println();

		final Collection<IBinarizationMethod> methods = loadFromFile(args[1], IBinarizationMethod.class);
		final Collection<IBinaryImageMetric> metrics = loadFromFile(args[2], IBinaryImageMetric.class);

		final String resultsPath = RESULTS_DIRNAME + extractName(filename) + "_" + CURRENT_TIME + FILE_SEPARATOR;
		final String averagesPath = resultsPath + AVERAGES_DIRNAME + FILE_SEPARATOR;

		createDirectory(resultsPath);
		createDirectory(averagesPath);

		Scanner input = null;
		try {
			input = new Scanner(new File(filename));
			final Results results = binarizeAndEvaluate(input, resultsPath, methods, metrics);
			printResults(filename, averagesPath, results, methods, metrics);

			for (RankingType type : RankingType.values()) {
				printRanking(filename, resultsPath, results, methods, metrics, type);
			}

		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} finally {
			if (input != null) {
				input.close();
			}
		}
	}

	public static <T> Collection<T> loadFromFile(String filename, Class<T> returnClass) {
		final Collection<T> result = new LinkedList<T>();

		Scanner input = null;
		try {
			input = new Scanner(new File(filename));
			while (input.hasNextLine()) {
				result.add(LoadManager.load(input.nextLine(), returnClass));
			}

		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (InstantiationException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(-1);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			System.exit(-1);
		} finally {
			if (input != null) {
				input.close();
			}
		}

		return result;
	}

	/**
	 * Cria um diretorio.
	 */
	public static void createDirectory(String name) {
		final File dir = new File(name);
		if (!dir.exists()) {
			dir.mkdir();
		}
	}

	/**
	 * Binariza cada imagem dada e a compara com o ground truth dado.
	 * A entrada deve possuir pares de arquivos, sendo o primeiro arquivo
	 * a imagem GT e o segundo a imagem em niveis de cinza a ser binarizada.
	 * As duas imagens sao carregadas, e a imagem niveis de cinza e binarizada
	 * por cada metodo e avaliado por cada metrica. Cada binarizacao gerara um
	 * arquivo com uma imagem binarizada e os resultados das metricas para
	 * aquela imagem. Por fim, as medias dos valores das metricas para cada metodo
	 * serao impressas em um arquivo no diretorio atual.
	 *
	 * @param input o leitor do arquivo de entrada com os nomes das imagens
	 * @param resultsPath o caminho do diretorio de resultados
	 * @param methods os metodos a ser utilizados
	 * @param metrics as metricas que avaliarao as binarizacoes
	 * @return o numero de avaliacoes feitas (numero de par de imagens)
	 * @throws IOException se um erro ocorrer
	 * @throws FileNotFoundException se o arquivo nao for encontrado
	 */
	public static Results binarizeAndEvaluate(Scanner input, String resultsPath,
			Collection<IBinarizationMethod> methods, Collection<IBinaryImageMetric> metrics)
			throws IOException, FileNotFoundException {

		final Results results = new Results(methods, metrics);

		while (input.hasNextLine()) {
			final Scanner line = new Scanner(input.nextLine());

			final String GTPath = line.next();
			final String grayPath = line.next();
			final String grayName = extractName(grayPath);

			final IBinaryImage groundTruth = Images.loadBinaryImage(GTPath);
			final IGrayLevelImage image = Images.loadGrayLevelImage(grayPath);

			/*para Pseudo F-Measure*/
			final IBinaryImage skeletonizedGT = (line.hasNext())
					? Images.loadBinaryImage(line.next())
					: Images.emptyBinaryImage();

			final String directory = resultsPath + grayName + FILE_SEPARATOR;

			createDirectory(directory);
			Images.saveImage(groundTruth, directory + grayName + "_GT.png");
			Images.saveImage(image, directory + grayName + ".png");

			for (IBinarizationMethod method : methods) {
				final String path = directory + method.getLabel();
				final String imagePath = path + ".png";
				final String textPath = path + ".txt";

				final IBinaryImage binarized = method.binarize(image);
				Images.saveImage(binarized, imagePath);

				final PrintWriter writer = new PrintWriter(new FileWriter(textPath), true);
				writer.println("Ground Truth: " + GTPath);
				writer.println("Imagem: " + grayPath);
				writer.println();

				for (IBinaryImageMetric metric : metrics) {
					final double evaluation = metric.evaluate(groundTruth, binarized, skeletonizedGT);

					results.addResult(method, metric, evaluation);
					writer.println(metric.getLabel() + ": " + metric.getFormat().format(evaluation));
				}
				writer.close();
			}

			System.out.println("Arquivos gerados para GT = " + GTPath + ", Imagem = " + grayPath);
		}

		return results;
	}

	/**
	 * Extrai o nome do arquivo sem sufixo do caminho dado. Se nao houver sufixo,
	 * ele nao alterara o fim do caminho.
	 * 
	 * @param path o caminho do arquivo
	 * @return o nome do arquivo sem sufixo
	 */
	public static String extractName(String path) {
		final int lastSeparator = path.lastIndexOf(FILE_SEPARATOR) + FILE_SEPARATOR.length();

		int lastDot = path.length() - 1;
		for (int i = path.length() - 1; i >= 0 && path.charAt(i) != '.'; i--) {
			lastDot = i;
		}
		lastDot = (lastDot == 0) ? path.length() : lastDot - 1;

		return path.substring(lastSeparator, lastDot);
	}

	/**
	 * Imprime as medias dos resultados dados obtidos pelas metricas.
	 * 
	 * @param inputFile o nome do arquivo de entrada
	 * @param averagesPath o caminho do diretorio de medias
	 * @param results os resultados das avaliacoes
	 * @param methods os metodos avaliados
	 * @param metrics as metricas utilizadas
	 * @throws IOException se um erro ocorrer
	 */
	public static void printResults(String inputFile, String averagesPath, Results results,
			Iterable<IBinarizationMethod> methods, Iterable<IBinaryImageMetric> metrics)
			throws IOException {

		for (IBinarizationMethod method : methods) {
			final String filename = averagesPath + method.getLabel() + ".txt";
			final PrintWriter writer = new PrintWriter(new FileWriter(filename), true);
			writer.println("Entrada: " + inputFile);
			writer.println("Método: " + method.toString());
			writer.println();

			for (IBinaryImageMetric metric : metrics) {
				final String result = metric.getFormat().format(results.getAverage(method, metric));
				writer.println(metric.getLabel() + ": " + result);
			}
			writer.close();
		}
	}

	/**
	 * Imprime a posicao de cada metodo de acordo com a avaliacao das metricas.
	 * 
	 * @param inputFile o nome do arquivo de entrada
	 * @param resultsPath o caminho do diretorio de resultados
	 * @param results os resultados das avaliacoes
	 * @param methods os metodos avaliados
	 * @param metrics as metricas utilizadas
	 * @param type o tipo de ranking utilizado
	 * @throws IOException se um erro ocorrer
	 */
	public static void printRanking(String inputFile, String resultsPath, Results results,
			Collection<IBinarizationMethod> methods, Iterable<IBinaryImageMetric> metrics,
			RankingType type)
			throws IOException {

		final Collection<Ranking> rankings = new LinkedList<Ranking>();

		for (IBinaryImageMetric metric : metrics) {
			rankings.addAll(type.newRankings(results, methods, metric));
		}

		final Ranking ranking = Ranking.finalRanking(rankings, methods);

		final PrintWriter writer = new PrintWriter(new FileWriter(resultsPath + type.filename()), true);
		writer.println("Entrada: " + inputFile);
		writer.println("Resultados:");
		writer.println();
		for (Ranking.Rank rank : ranking) {
			writer.printf("%2d. %4.0f pontos:  %s %n", rank.value(), rank.score(), rank.method().toString());
		}
		writer.close();
	}
}
