package binarization;

import binarization.util.Comparators;
import binarization.data.util.Pair;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Classe que representa um ranking de metodos. Seu conteudo e obtido atraves da
 * iteracao sobre os ranks. O iterador devolvido nao pode alterar o ranking.
 *
 * @author denis
 * @see Results
 * @see IBinarizationMethod
 * @see IBinaryImageMetric
 */
public class Ranking implements Iterable<Ranking.Rank> {

	private final List<Rank> ranks;

	/**
	 * Constroi um ranking com a view imutavel da lista de ranks dada.
	 * Os metodos de fabrica devem ser usados no lugar deste construtor.
	 * 
	 * @param ranks uma lista com os ranks
	 */
	private Ranking(List<Rank> ranks) {
		this.ranks = Collections.unmodifiableList(ranks);
	}

	@Override
	public Iterator<Rank> iterator() {
		return ranks.iterator();
	}

	/**
	 * Classe encaixada que define uma estrutura simples de rank.
	 * Feito para simplificar a iteracao, que originalmente utilizava pares.
	 */
	public static class Rank {

		private final IBinarizationMethod method;
		private final int value;
		private final double score;

		private Rank(IBinarizationMethod method, int value, double score) {
			this.method = method;
			this.value = value;
			this.score = score;
		}

		/**
		 * Devolve o valor do rank.
		 * 
		 * @return o valor do rank
		 */
		public int value() {
			return value;
		}

		/**
		 * Devolve o metodo associado a esse rank.
		 * 
		 * @return o metodo associado a esse rank
		 */
		public IBinarizationMethod method() {
			return method;
		}

		public double score() {
			return score;
		}
	}

	/**
	 * Devolve um novo ranking em relacao a metrica dada, a partir
	 * da media dos resultados dados, para cada metodo dado.
	 * 
	 * @param results os resultados que derivarao o ranking
	 * @param methods os metodos avaliados
	 * @param metric a metrica que avaliou
	 * @return um novo ranking em relacao a metrica dada
	 */
	public static Ranking newAverageRanking(Results results,
			Collection<IBinarizationMethod> methods, IBinaryImageMetric metric) {

		final List<Pair<IBinarizationMethod, Double>> list = new ArrayList<Pair<IBinarizationMethod, Double>>(methods.size());

		for (IBinarizationMethod method : methods) {
			final Double value = Double.valueOf(results.getAverage(method, metric));
			list.add(Pair.of(method, value));
		}

		final PairSecondComparator<Double> comparator = new PairSecondComparator<Double>(metric.getResultComparator());
		Collections.sort(list, Collections.reverseOrder(comparator));	//a ordenacao e decrescente

		return rankingOf(list);
	}

	/**
	 * Devolve uma colecao com novos rankings em relacao a metrica dada, a partir
	 * de cada resultado dados, para cada metodo dado.
	 * 
	 * @param results os resultados que derivarao o ranking
	 * @param methods os metodos avaliados
	 * @param metric a metrica que avaliou
	 * @return uma colecao com novos rankings em relacao a metrica dada
	 */
	public static Collection<Ranking> newPerImageRankings(Results results,
			Collection<IBinarizationMethod> methods, IBinaryImageMetric metric) {

		final Map<IBinarizationMethod, List<Double>> map = new HashMap<IBinarizationMethod, List<Double>>(methods.size());
		for (IBinarizationMethod method : methods) {
			map.put(method, results.getResults(method, metric));
		}

		final Collection<Ranking> result = new ArrayList<Ranking>();

		final int size = methods.size() > 0
				? map.get(methods.iterator().next()).size()
				: 0;

		for (int index = 0; index < size; index++) {
			final List<Pair<IBinarizationMethod, Double>> list = new ArrayList<Pair<IBinarizationMethod, Double>>(methods.size());

			for (IBinarizationMethod method : methods) {
				final Double value = Double.valueOf(map.get(method).get(index));
				list.add(Pair.of(method, value));
			}
			final PairSecondComparator<Double> comparator = new PairSecondComparator<Double>(metric.getResultComparator());
			Collections.sort(list, Collections.reverseOrder(comparator));	//a ordenacao e decrescente

			result.add(rankingOf(list));
		}

		return result;
	}

	/**
	 * Devolve o ranking obtido a partir da soma dos rankings dados, em relacao
	 * aos metodos dados. Mais precisamente, os rankings de cada metodo sao somados,
	 * e, apos a soma, eles sao ordenados do menor para o maior. Baseado nesta
	 * ordenacao, um novo ranking e criado e devolvido.
	 * 
	 * @param rankings os rankings a ser acumulados
	 * @param methods os metodos avaliados
	 * @return o ranking obtido a partir da soma dos rankings dados
	 */
	public static Ranking finalRanking(Iterable<Ranking> rankings, Collection<IBinarizationMethod> methods) {
		final Map<IBinarizationMethod, Integer> acc = accumulateRankings(rankings, methods);

		final List<Pair<IBinarizationMethod, Double>> list = new ArrayList<Pair<IBinarizationMethod, Double>>(methods.size());

		for (Map.Entry<IBinarizationMethod, Integer> entry : acc.entrySet()) {
			final IBinarizationMethod method = entry.getKey();
			final Double value = Double.valueOf(entry.getValue().doubleValue());
			list.add(Pair.of(method, value));
		}

		final PairSecondComparator<Double> comparator = new PairSecondComparator<Double>(Comparators.INCREASING);
		Collections.sort(list, comparator);

		return rankingOf(list);
	}

	/**
	 * Devolve o ranking obtido a partir da lista de pares ordenada dada. Os pares
	 * de mesmo valor obterao o mesmo rank.
	 * 
	 * @param pairs os pares metodo-valor da qual sera obtido o rank
	 * @return o ranking obtido a partir da lista de pares ordenada dada
	 */
	private static Ranking rankingOf(List<Pair<IBinarizationMethod, Double>> pairs) {
		final List<Rank> ranks = new ArrayList<Rank>(pairs.size());

		int rank = 1;
		Double old = pairs.get(0).second();
		for (Pair<IBinarizationMethod, Double> pair : pairs) {
			if (!old.equals(pair.second())) {
				rank++;
				old = pair.second();
			}
			ranks.add(new Rank(pair.first(), rank, pair.second()));
		}

		return new Ranking(ranks);
	}

	/**
	 * Devolve um mapa dos metodos para seus ranks acumulados, obtido a partir dos
	 * rankings dados. Utiliza a colecao de metodos para inicializar o mapa.
	 * 
	 * @param rankings os rankings a ser acumulados
	 * @param methods todos os metodos nos rankings
	 * @return um mapa dos metodos para seus ranks acumulados
	 */
	private static Map<IBinarizationMethod, Integer> accumulateRankings(Iterable<Ranking> rankings,
			Collection<IBinarizationMethod> methods) {

		final Map<IBinarizationMethod, Integer> acc = new HashMap<IBinarizationMethod, Integer>(methods.size());
		for (IBinarizationMethod method : methods) {
			acc.put(method, Integer.valueOf(0));
		}

		for (Ranking ranking : rankings) {
			for (Rank rank : ranking.ranks) {
				final int current = acc.get(rank.method).intValue();
				final int sum = current + rank.value;
				acc.put(rank.method, Integer.valueOf(sum));
			}
		}

		return acc;
	}

	/**
	 * Classe utilitaria que compara pares baseado utilizando um comparador para
	 * o segundo elemento apenas.
	 * 
	 * @param <T> o tipo do segundo elemento do par
	 */
	private static class PairSecondComparator<T> implements Comparator<Pair<?, T>> {

		private final Comparator<T> comparator;

		/**
		 * Constroi um comparador de pares a partir do comparador do segundo elemento
		 * dado.
		 * 
		 * @param comparator um comparador para o segundo elemento
		 */
		public PairSecondComparator(Comparator<T> comparator) {
			this.comparator = comparator;
		}

		@Override
		public int compare(Pair<?, T> o1, Pair<?, T> o2) {
			return comparator.compare(o1.second(), o2.second());
		}
	}
}