package binarization.data;

/**
 * Implementacao esqueletal da imagens niveis de cinza. Somente os acessores
 * dos pixels e do tamanho devem ser definidos.
 *
 * @author denis
 * @see IGrayLevelImage
 * @see AbstractImage
 * @see AbstractBinaryImage
 */
public abstract class AbstractGrayLevelImage extends AbstractImage implements IGrayLevelImage {

	@Override
	public abstract ImageSize size();

	@Override
	public abstract Pixel pixelAtUnchecked(int x, int y);

	@Override
	public IGrayLevelImage windowOf(ImagePoint center, ImageSize size) {
		return new GrayLevelWindow(super.windowOf(center, size));
	}

	@Override
	public IGrayLevelImage inverse() {
		return new GrayLevelInverse(this);
	}

	@Override
	public IHistogram getHistogram() {
		final long[] frequencies = new long[Pixel.ofMaxGrayLevel().value() + 1];

		for (int i = 0; i < frequencies.length; i++) {
			frequencies[i] = 0;
		}

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {

				frequencies[pixelAtUnchecked(x, y).value()] += 1;

			}
		}

		return new SimpleHistogram(frequencies);
	}

	@Override
	public int getMinimum() {
		int min = Pixel.ofMaxGrayLevel().value();

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {

				final int pixel = pixelAtUnchecked(x, y).value();
				if (min > pixel) {
					min = pixel;
				}

			}
		}

		return min;
	}

	@Override
	public int getMaximum() {
		int max = 0;

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {

				final int pixel = pixelAtUnchecked(x, y).value();
				if (max < pixel) {
					max = pixel;
				}

			}
		}

		return max;
	}

	@Override
	public double getMean() {
		long sum = 0;

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {

				sum += pixelAtUnchecked(x, y).value();

			}
		}

		return (double) sum / size().numberOfPixels();
	}

	@Override
	public double getStandardDeviation() {
		final double mean = getMean();
		double sum = 0;

		for (int x = 0; x < size().width(); x++) {
			for (int y = 0; y < size().height(); y++) {

				final double diff = pixelAtUnchecked(x, y).value() - mean;
				sum += diff * diff;

			}
		}

		final double variance = sum / size().numberOfPixels();

		return Math.sqrt(variance);
	}

	/**
	 * Janela de uma imagem em niveis de cinzs. Utiliza a janela da superclasse.
	 * Esta classe e definida somente para que possa herdar desta classe abstrata.
	 */
	private static class GrayLevelWindow extends AbstractGrayLevelImage {

		private final IImage window;

		private GrayLevelWindow(IImage window) {
			this.window = window;
		}

		@Override
		public ImageSize size() {
			return window.size();
		}

		@Override
		public Pixel pixelAtUnchecked(int x, int y) {
			return window.pixelAtUnchecked(x, y);
		}
	}

	private static class GrayLevelInverse extends AbstractGrayLevelImage {

		private final IGrayLevelImage original;

		private GrayLevelInverse(IGrayLevelImage original) {
			this.original = original;
		}

		@Override
		public ImageSize size() {
			return original.size();
		}

		@Override
		public Pixel pixelAtUnchecked(int x, int y) {
			return Pixel.of(Pixel.ofMaxGrayLevel().value() - original.pixelAtUnchecked(x, y).value());
		}

		@Override
		public IGrayLevelImage inverse() {
			return original;
		}
	}
}
