package binarization.data;

/**
 * Implementacao esqueletal da imagens. Somente os acessores
 * dos pixels e do tamanho devem ser definidos.
 *
 * @author denis
 * @see IImage
 * @see AbstractBinaryImage
 * @see AbstractGrayLevelImage
 */
public abstract class AbstractImage implements IImage {

	@Override
	public boolean isEmpty() {
		return size().isEmpty();
	}

	@Override
	public abstract ImageSize size();

	@Override
	public Pixel pixelAt(ImagePoint p) {
		if (!p.isInside(size())) {
			throw new IndexOutOfBoundsException("Point p:" + p + " is not inside this image.");
		}
		return pixelAtUnchecked(p.x(), p.y());
	}

	@Override
	public abstract Pixel pixelAtUnchecked(int x, int y);

	@Override
	public IImage windowOf(ImagePoint center, ImageSize size) {
		if (!center.isInImage(this)) {
			throw new IndexOutOfBoundsException("Central point must be inside the original image.");
		}
		if (!hasOddDimensions(size)) {
			throw new IllegalArgumentException("Size dimensions are not odd.");
		}

		final int xTrans = center.x() - (size.width() / 2);
		final int yTrans = center.y() - (size.height() / 2);

		final ImagePoint trans = ImagePoint.of(
				xTrans > 0 ? xTrans : 0,
				yTrans > 0 ? yTrans : 0);

		final int realWidth = realDimension(xTrans, size.width(), this.size().width());
		final int realHeight = realDimension(yTrans, size.height(), this.size().height());

		return new Window(this, trans, ImageSize.of(realWidth, realHeight));
	}

	/**
	 * Devolve a dimensao real da janela. Caso a janela esteja totalmente
	 * contida na imagem, esta dimensao e igual aa dimensao dada. Caso contrario,
	 * a parte inicial ou final sera corrigida e o valor real devolvido.
	 * 
	 * @param start a posicao inicial da janela
	 * @param dimension a dimensao requisitada da janela
	 * @param imageDimension a dimensao total da imagem
	 * @return a dimensao real da janela
	 */
	private int realDimension(int start, int dimension, int imageDimension) {
		final int excess = start + dimension - imageDimension;
		final int startCorrection = (start >= 0) ? 0 : start;
		final int endCorrection = (excess < 0) ? 0 : -excess;

		return dimension + startCorrection + endCorrection;
	}

	/**
	 * Devolve {@code true} se o tamanho dado possuir dimensoes impares.
	 * 
	 * @param size o tamanho a ser verificado
	 * @return {@code true} se o tamanho dado possuir dimensoes impares
	 */
	private boolean hasOddDimensions(ImageSize size) {
		return !isEven(size.width()) && !isEven(size.height());
	}

	/**
	 * Devolve {@code true} se o valor dado for par.
	 * 
	 * @param value o valor a ser verificado
	 * @return {@code true} se o valor dado for par
	 */
	private boolean isEven(int value) {
		return (value % 2) == 0;
	}

	/**
	 * Janela de uma imagem. Nao copia os pixels, apenas desloca
	 * o inicio para que ela possa ser interpretada como uma imagem comum.
	 */
	private static class Window extends AbstractImage {

		private final IImage original;
		private final int xStart;
		private final int yStart;
		private final ImageSize size;

		/**
		 * Constroi uma janela da imagem dada, com o ponto inicial e tamanho dados.
		 * 
		 * @param original a imagem original da janela
		 * @param start a posicao inicial
		 * @param size o tamanho da janela
		 */
		private Window(IImage original, ImagePoint start, ImageSize size) {
			this.original = original;
			this.xStart = start.x();
			this.yStart = start.y();
			this.size = size;
		}

		@Override
		public ImageSize size() {
			return size;
		}

		@Override
		public Pixel pixelAtUnchecked(int x, int y) {
			return original.pixelAtUnchecked(xStart + x, yStart + y);
		}
	}
}
