package binarization.data;

/**
 * O tamanho de uma imagem.
 *
 * @author denis
 * @see IImage
 * @see IBinaryImage
 * @see IGrayLevelImage
 */
public class ImageSize {

	private static final ImageSize EMPTY = new ImageSize(0, 0);
	private final int width;
	private final int height;

	private ImageSize(int width, int height) {
		this.width = width;
		this.height = height;
	}

	/**
	 * Devolve o tamanho de uma imagem de largura e altura dadas.
	 * 
	 * @param width a largura da imagem
	 * @param height a altura da imagem
	 * @return o tamanho de uma imagem de largura e altura dadas
	 * @throws IllegalArgumentException caso uma das dimensoes seja nula ou negativa
	 */
	public static ImageSize of(int width, int height) {
		if (width <= 0 || height <= 0) {
			throw new IllegalArgumentException("Size dimensions must be positive.");
		}
		return new ImageSize(width, height);
	}

	/**
	 * Devolve o tamanho de uma imagem quadrada de lado dado.
	 * 
	 * @param sides o tamanho dos lados da imagem quadrada
	 * @return o tamanho de uma imagem quadrada de lado dado
	 * @throws IllegalArgumentException caso o tamanho seja nulo ou negativo
	 */
	public static ImageSize of(int sides) {
		return of(sides, sides);
	}

	/**
	 * Devolve o tamanho vazio. Sempre devolve o mesmo tamanho.
	 * 
	 * @return o tamanho vazio
	 */
	public static ImageSize ofEmpty() {
		return EMPTY;
	}

	/**
	 * Devolve a largura de uma imagem.
	 * 
	 * @return a largura de uma imagem
	 */
	public int width() {
		return width;
	}

	/**
	 * Devolve a altura de uma imagem.
	 * 
	 * @return a altura de uma imagem
	 */
	public int height() {
		return height;
	}

	/**
	 * Devolve o numero de pixels da imagem.
	 * 
	 * @return o numero de pixels da imagem
	 */
	public int numberOfPixels() {
		return width * height;
	}

	/**
	 * Devolve {@code true} se o tamanho for vazio.
	 * 
	 * @return {@code true} se o tamanho for vazio.
	 */
	public boolean isEmpty() {
		return (width <= 0 || height <= 0);
	}

	@Override
	public boolean equals(Object o) {
		if (!(o instanceof ImageSize)) {
			return false;
		}

		final ImageSize that = (ImageSize) o;

		return (this.width == that.width && this.height == that.height);
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 71 * hash + this.width;
		hash = 71 * hash + this.height;
		return hash;
	}

	@Override
	public String toString() {
		return "ImageSize width:" + width + " height:" + height;
	}
}