package binarization.data;

import binarization.util.ImageIoUtil;

/**
 * Classe utilitaria nao-instanciavel que prove metodos estaticos auxiliares
 * para as imagens.
 *
 * <p>Unica ponte externa entre as interfaces e as suas respectivas
 * implementacoes padrao.</p>
 *
 * @author denis
 * @see IBinaryImage
 * @see IGrayLevelImage
 * @see ImageSize
 */
public class Images {

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private Images() {
	}

	/**
	 * Devolve uma imagem binaria vazia. Todas as chamadas a este metodo sempre
	 * devolverao a mesma imagem.
	 * 
	 * @return uma imagem binaria vazia
	 */
	public static IBinaryImage emptyBinaryImage() {
		return EmptyBinaryImage.INSTANCE;
	}

	/**
	 * Devolve uma imagem em niveis de cinza vazia. Todas as chamadas a este
	 * metodo sempre devolverao a mesma imagem.
	 * 
	 * @return uma imagem em niveis de cinza vazia
	 */
	public static IGrayLevelImage emptyGrayLevelImage() {
		return EmptyGrayLevelImage.INSTANCE;
	}

	/**
	 * Cria uma imagem binaria a partir do arquivo de nome dado.
	 * O arquivo da imagem e fechado apos seu carregamento.
	 *
	 * @param filename o nome do arquivo a ser aberto
	 * @return uma imagem binaria obtida do arquivo de nome dado
	 */
	public static IBinaryImage loadBinaryImage(String filename) {
		final int[][] intensities = ImageIoUtil.loadGrayLevelImageFile(filename);
		final ImageSize size = ImageSize.of(intensities.length, intensities[0].length);

		final IBinaryImage.Builder builder = new SimpleBinaryImageBuilder(size);

		final int black = findMinimum(intensities);

		for (int x = 0; x < size.width(); x++) {
			for (int y = 0; y < size.height(); y++) {

				if (intensities[x][y] == black) {
					builder.setPixelUnchecked(x, y);
				}

			}
		}

		return builder.build();
	}

	/**
	 * Devolve o valor minimo da matriz de intensidades dada.
	 * 
	 * @param intensities a matriz de intensidades
	 * @return o valor minimo da matriz de intensidades dada
	 */
	private static int findMinimum(int[][] intensities) {
		int minimum = intensities[0][0];

		for (int x = 0; x < intensities.length; x++) {
			for (int y = 0; y < intensities[0].length; y++) {

				if (minimum > intensities[x][y]) {
					minimum = intensities[x][y];
				}

			}
		}
		return minimum;
	}

	/**
	 * Cria uma imagem em niveis de cinza a partir do arquivo de nome dado.
	 * O arquivo da imagem e fechado apos seu carregamento.
	 *
	 * @param filename o nome do arquivo a ser aberto
	 * @return uma imagem em niveis de cinza obtida do arquivo de nome dado
	 */
	public static IGrayLevelImage loadGrayLevelImage(String filename) {
		final int[][] intensities = ImageIoUtil.loadGrayLevelImageFile(filename);
		final ImageSize size = ImageSize.of(intensities.length, intensities[0].length);

		final IGrayLevelImage.Builder builder = new SimpleGrayLevelImageBuilder(size);

		for (int x = 0; x < size.width(); x++) {
			for (int y = 0; y < size.height(); y++) {

				builder.setPixelUnchecked(x, y, Pixel.ofRestricted(intensities[x][y]));

			}
		}

		return builder.build();
	}

	/**
	 * Grava a imagem dada no arquivo de nome dado.
	 * 
	 * @param image a imagem a ser gravada
	 * @param filename o arquivo no qual a imagem sera gravada (pode nao existir)
	 */
	public static void saveImage(IImage image, String filename) {
		final int width = image.size().width();
		final int height = image.size().height();

		final int[][] pixels = new int[width][height];

		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {

				pixels[x][y] = image.pixelAtUnchecked(x, y).value();

			}
		}

		ImageIoUtil.saveGrayLevelImageFile(pixels, filename);
	}

	/**
	 * Cria um construtor para uma imagem binaria de tamanho dado na qual os
	 * objetos sao pretos e o fundo e branco.
	 * 
	 * @param size o tamanho da imagem
	 * @return um construtor de uma imagem binaria
	 */
	public static IBinaryImage.Builder newBinaryImageBuilder(ImageSize size) {
		return new SimpleBinaryImageBuilder(size);
	}

	/**
	 * Cria um construtor para uma imagem em niveis de cinza de tamanho dado.
	 * 
	 * @param size o tamanho da imagem
	 * @return um construtor de uma imagem em niveis de cinza
	 */
	public static IGrayLevelImage.Builder newGrayLevelImageBuilder(ImageSize size) {
		return new SimpleGrayLevelImageBuilder(size);
	}
}
