package binarization.data;

/**
 * Implementacao simples de um construtor de imagem binaria. Por padrao, o fundo
 * e branco e os objetos sao pretos.
 * 
 * <p>Nao e imutavel, claro, mas sua mutabilidade se restringe a pintar os pixels
 * do objeto, nao podendo apagar o que ja foi pintado. Nao pode ser instanciado
 * para forcar a utilizar os metodos de fabrica na classe utilitaria {@link Images}</p>.
 *
 * @author denis
 */
class SimpleBinaryImageBuilder implements IBinaryImage.Builder {

	private final Pixel[][] pixels;
	private final ImageSize size;

	/**
	 * Constroi um construtor de imagem binaria de tamanho dado.
	 * A cor de fundo e branca.
	 * 
	 * @param size o tamanho da imagem a ser construida
	 */
	SimpleBinaryImageBuilder(ImageSize size) {
		if (size.isEmpty()) {
			throw new IllegalArgumentException("Size must be positive.");
		}
		this.pixels = initializePixels(size);
		this.size = size;
	}

	/**
	 * Inicializa os pixels como fundo.
	 * 
	 * @param size o tamanho da imagem
	 * @return os pixels inicializados como fundo
	 */
	private Pixel[][] initializePixels(ImageSize size) {
		final Pixel[][] background = new Pixel[size.width()][size.height()];

		for (int x = 0; x < size.width(); x++) {
			for (int y = 0; y < size.height(); y++) {
				background[x][y] = Pixel.ofWhite();
			}
		}

		return background;
	}

	@Override
	public void setPixel(ImagePoint p) {
		if (!p.isInside(size)) {
			throw new IndexOutOfBoundsException("Point p:" + p + " is not inside this builder.");
		}
		pixels[p.x()][p.y()] = Pixel.ofBlack();
	}

	@Override
	public void setPixelUnchecked(int x, int y) {
		pixels[x][y] = Pixel.ofBlack();
	}

	@Override
	public IBinaryImage build() {
		return new SimpleBinaryImage(pixels);
	}

	boolean isSet(int x, int y) {
		return pixels[x][y] == Pixel.ofBlack();
	}
}
