package binarization.data.util;

import binarization.data.IBinaryImage;
import binarization.data.IGrayLevelImage;
import binarization.data.ImageSize;
import binarization.data.Images;
import binarization.data.Pixel;
import ij.process.BinaryProcessor;
import ij.process.ByteProcessor;
import ij.process.ImageProcessor;
import ij.process.ShortProcessor;

/**
 *
 * @author
 */
public class ImageOperations {

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private ImageOperations() {
	}

	public static IBinaryImage outline(IBinaryImage input) {
		final BinaryProcessor processor = convertToBinaryProcessor(input);
		processor.outline();
		return convertToBinary(processor);
	}

	public static GridMarker euclidianDistanceMap(IBinaryImage input) {
		final BinaryProcessor processor = convertToBinaryProcessor(input);

		return doEdm(processor);
	}
	
	static GridMarker doEdm(BinaryProcessor processor) {
		final ShortProcessor map = new ij.plugin.filter.EDM().make16bitEDM(processor);

		final ImageSize size = ImageSize.of(processor.getWidth(), processor.getHeight());
		final GridMarker result = new GridMarker(size, 0);
		for (int x = 0; x < processor.getWidth(); x++) {
			for (int y = 0; y < processor.getHeight(); y++) {

				result.set(x, y, map.get(x, y) & 0xFFFF);

			}
		}

		return result;
	}

	/**
	 * Devolve uma imagem em niveis de cinza equivalente a imagem dada do ImageJ.
	 * 
	 * @param input a imagem da qual se quer uma equivalente
	 * @return uma imagem em niveis de cinza equivalente a imagem dada do ImageJ
	 */
	static IGrayLevelImage convertToGrayLevel(ImageProcessor input) {
		final ImageSize size = ImageSize.of(input.getWidth(), input.getHeight());
		final IGrayLevelImage.Builder builder = Images.newGrayLevelImageBuilder(size);

		final ImageProcessor converted = input.convertToByte(true);
		final byte[] pixels = (byte[]) converted.getPixels();
		for (int x = 0; x < size.width(); x++) {
			for (int y = 0; y < size.height(); y++) {
				final int offset = y * size.width() + x;
				final int value = pixels[offset] & 0xFF;

				builder.setPixelUnchecked(x, y, Pixel.of(value));
			}
		}

		return builder.build();
	}

	/**
	 * Devolve uma imagem binaria equivalente a imagem dada do ImageJ.
	 * 
	 * @param input a imagem da qual se quer uma equivalente
	 * @return uma imagem binaria equivalente a imagem dada do ImageJ
	 */
	static IBinaryImage convertToBinary(ImageProcessor input) {
		final ImageSize size = ImageSize.of(input.getWidth(), input.getHeight());
		final IBinaryImage.Builder builder = Images.newBinaryImageBuilder(size);

		final ImageProcessor converted = input.convertToByte(true);
		final byte[] pixels = (byte[]) converted.getPixels();
		for (int x = 0; x < size.width(); x++) {
			for (int y = 0; y < size.height(); y++) {
				final int offset = y * size.width() + x;
				final int value = pixels[offset] & 0xFF;

				if (value == 0) {
					builder.setPixelUnchecked(x, y);
				}
			}
		}

		return builder.build();
	}

	/**
	 * Devolve uma imagem em niveis de cinza do ImageJ equivalente aa imagem dada.
	 * 
	 * @param input a imagem da qual se quer uma equivalente
	 * @return uma imagem em niveis de cinza do ImageJ equivalente aa imagem dada
	 */
	static ByteProcessor convertToByteProcessor(IGrayLevelImage input) {
		final ByteProcessor result = new ByteProcessor(input.size().width(), input.size().height());

		for (int x = 0; x < input.size().width(); x++) {
			for (int y = 0; y < input.size().height(); y++) {

				result.set(x, y, input.pixelAtUnchecked(x, y).value());

			}
		}

		return result;
	}

	/**
	 * Devolve uma imagem binaria do ImageJ equivalente aa imagem dada.
	 * 
	 * @param input a imagem da qual se quer uma equivalente
	 * @return uma imagem binaria do ImageJ equivalente aa imagem dada
	 */
	static BinaryProcessor convertToBinaryProcessor(IBinaryImage input) {
		final BinaryProcessor result = new BinaryProcessor(new ByteProcessor(input.size().width(), input.size().height()));

		for (int x = 0; x < input.size().width(); x++) {
			for (int y = 0; y < input.size().height(); y++) {

				result.set(x, y, input.pixelAtUnchecked(x, y).isBlack() ? 0 : 255);

			}
		}

		return result;
	}
}
