package binarization.examples;

import binarization.IBinaryImageMetric;
import binarization.data.IBinaryImage;
import binarization.data.Images;
import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

/**
 * Programa que avalia as imagens dadas (GT, binarizada, GT esqueleto)atraves
 * de cada metrica implementada, e tira a media de todas as avaliacoes.
 * 
 * <p>E verificado se o GT esqueleto sera utilizado na primeira linha da entrada,
 * assumindo que havera outros em cada outra linha do programa. Se nao houver GT
 * esqueleto ne primeira linha, somente as metricas que nao a usam serao utilizadas.
 * Caso ela esteja na primeira linha, mas nao haja GT esqueleto em algumas das
 * restantes, o valor das metricas que utilizam a GT esqueleto e indefinido.</p>
 * 
 * @author denis
 */
public class ExampleMetricsAverage {

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private ExampleMetricsAverage() {
	}

	/**
	 * Ponto de entrada do programa. Imprime o resultado na saida padrao.
	 * 
	 * @param args arquivo que contem o nome dos arquivos das 2 ou 3 imagens a cada linha
	 */
	public static void main(String[] args) {
		if (args.length < 1) {
			System.out.println("Requer um arquivo com as imagens de entrada.");
			System.exit(-1);
		}

		System.out.println("Entrada: " + args[0]);
		System.out.println();

		final Collection<IBinaryImageMetric> metrics = (hasSkeletonizedGroundTruth(args[0]))
				? Metrics.getAllMetricsWithPFM()
				: Metrics.getAllMetrics();

		final Map<IBinaryImageMetric, Double> sum = new HashMap<IBinaryImageMetric, Double>(metrics.size());
		for (IBinaryImageMetric metric : metrics) {
			sum.put(metric, 0.0);
		}

		Scanner scanner = null;
		try {
			scanner = new Scanner(new File(args[0]));

			int evaluations = 0;
			while (scanner.hasNextLine()) {
				final Scanner line = new Scanner(scanner.nextLine());
				final IBinaryImage groundTruth = Images.loadBinaryImage(line.next());
				final IBinaryImage binarized = Images.loadBinaryImage(line.next());

				final IBinaryImage skeletonizedGT = (line.hasNext())
						? Images.loadBinaryImage(line.next())
						: Images.emptyBinaryImage();

				for (IBinaryImageMetric metric : metrics) {
					final double evaluation = metric.evaluate(groundTruth, binarized, skeletonizedGT);
					sum.put(metric, sum.get(metric) + evaluation);
				}
				evaluations++;
			}

			for (IBinaryImageMetric metric : metrics) {
				final double average = sum.get(metric) / evaluations;
				final String message = metric.getFormat().format(average);

				System.out.println(metric.getLabel() + ": " + message);
			}
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} finally {
			if (scanner != null) {
				scanner.close();
			}
		}
	}

	/**
	 * Verifica se ha GT esqueleto na primeira linha da entrada.
	 * 
	 * @param filename o arquivo de entrada
	 * @return {@code true} se houver GT esqueleto na primeira linha da entrada
	 */
	private static boolean hasSkeletonizedGroundTruth(String filename) {
		boolean hasSkeletonized = false;
		Scanner scanner = null;
		try {
			scanner = new Scanner(new File(filename));
			final Scanner line = new Scanner(scanner.nextLine());
			line.next();
			line.next();
			hasSkeletonized = line.hasNext();

		} catch (IOException e) {
			e.printStackTrace();
			System.exit(-1);
		} finally {
			if (scanner != null) {
				scanner.close();
			}
		}

		return hasSkeletonized;
	}
}
