package binarization.methods;

import binarization.IBinarizationMethod;
import binarization.data.IGrayLevelImage;

/**
 * Metodo de Niblack. (precisa de citacao)
 * 
 * <p>Calcula o limiar para uma janela igual a 
 * {@code m(x, y) + k * s(x, y)}, onde:<ol>
 * 
 * <li>{@code k} e o peso do desvio padrao,</li>
 * <li>{@code m(x, y) e s(x, y)} sao a media e o desvio padrao da janela
 * centrada no pixel {@code (x, y)}, respectivamente.</li>
 * </ol></p>
 * 
 * <p>O valor recomendado de {@code k = -0.2} para objetos pretos.</p>
 *
 * @author denis
 * @see SauvolaMethod
 */
public class NiblackMethod extends AbstractLocalAdaptiveThresholding implements IBinarizationMethod {

	private static final double RECOMMENDED_WEIGHT = -0.2;
//
	private final double weight;

	/**
	 * Metodo de Niblack com tamanho de janela dado e peso k dado.
	 * 
	 * @param windowSize o tamanho da janela quadrada
	 * @param weight o peso do desvio padrao k
	 */
	public NiblackMethod(int windowSize, double weight) {
		super(windowSize);
		this.weight = weight;
	}

	/**
	 * Metodo de Niblack com tamanho de janela dado e peso k recomendado.
	 * 
	 * @param windowSize o tamanho da janela quadrada
	 */
	public NiblackMethod(int windowSize) {
		this(windowSize, RECOMMENDED_WEIGHT);
	}

	@Override
	public double getThreshold(IGrayLevelImage window) {
		final double mean = window.getMean();
		final double std = window.getStandardDeviation();
		return mean + weight * std;
	}

	@Override
	public String getLabel() {
		return "niblack_window" + getWindowSize() + "_k" + weight;
	}

	@Override
	public String toString() {
		return "Niblack, Window Size: " + getWindowSize() + ", k: " + weight;
	}
}
