package binarization.methods;

import binarization.IBinarizationMethod;
import binarization.data.IAccumulatedHistogram;
import binarization.data.IGrayLevelImage;
import binarization.data.INormalizedHistogram;

/**
 * Metodo de Otsu. (precisa de citacao)
 * 
 * <p>Define:<ol>
 * 
 * <li>{@code omega(k) = valor da intensidade k do histograma acumulado de ordem zero} </li>
 * <li>{@code mu(k) = valor da intensidade k do histograma acumulado de ordem um} </li>
 * <li>{@code muT = valor da maior intensidade do histograma acumulado de ordem um} </li>
 * <li>{@code variancia interclasse = (muT * omega(k) - mu(k))^2 / omega(k) * (1 - omega(k))} </li>
 * </ol></p>
 * 
 * <p>O histograma, definido desta forma, e o normalizado.
 * (distribuicao de probabilidades).</p>
 * 
 * <p>O valor do limiar e aquele dentre todas as intensidades que maximiza
 * a variancia interclasse.</p>
 * 
 * <p>Este metodo e nao-parametrico, portanto, e feito de forma imutavel. Como
 * nao possui estado, nao precisa de mais meios de seguranca.</p>
 * 
 * @author denis
 */
public class OtsuMethod extends AbstractGlobalThresholding implements IBinarizationMethod {

	public static final OtsuMethod INSTANCE = new OtsuMethod();

	private OtsuMethod() {
	}

	@Override
	public double getThreshold(IGrayLevelImage image) {
		final INormalizedHistogram normalized = image.getHistogram().normalize();
		final IAccumulatedHistogram omega = normalized.accumulate(0);
		final IAccumulatedHistogram mu = normalized.accumulate(1);

		final double muTotal = mu.valueAt(mu.maxIntensity());

		int threshold = 0;
		double maxBetweenClassVariance = 0;
		for (int k = 0; k < mu.maxIntensity(); k++) {

			final double numerator = muTotal * omega.valueAt(k) - mu.valueAt(k);
			final double denomunator = omega.valueAt(k) * (1.0 - omega.valueAt(k));

			final double betweenClassVariance = (numerator * numerator) / denomunator;

			if (maxBetweenClassVariance < betweenClassVariance) {
				maxBetweenClassVariance = betweenClassVariance;
				threshold = k;
			}

		}

		/*o valor definido no paper e inteiro. Esta soma tenta munimuzar erros de precisao*/
		return threshold + Double.MIN_NORMAL;
	}

	@Override
	public String getLabel() {
		return "otsu";
	}

	@Override
	public String toString() {
		return "Otsu";
	}
}