package binarization.methods;

import binarization.IBinarizationMethod;
import binarization.data.ContrastType;
import binarization.data.IBinaryImage;
import binarization.data.IGrayLevelImage;
import binarization.data.Images;
import binarization.data.MaskedImage;
import binarization.data.ImageSize;
import binarization.data.ImagePoint;

/**
 * Metodo de Su, Lu e Tan. (precisa de citacao)
 * 
 * <p>Este metodo e dividido em tres partes: <ol>
 * 
 * <li>Obtencao da imagem de contraste (em niveis de cinza)</li>
 * <li>Identificacao dos pixels de alto contraste (numa imagem binaria)</li>
 * <li>Binarizacao adaptativa local da imagem de entrada</li>
 * </ol></p>
 * 
 * <p>A imagem de contraste e obtida atraves do maximo e do minimo de uma janela
 * da imagem original. O tamanho recomendado desta janela e 3x3. Cada pixel da
 * imagem de contraste e calculado da seguinte forma a partir da janela centrada
 * nele: <ol>
 * 
 * <li>{@code D(x, y) = (fmax(x, y) - fmin(x, y)) / (fmax(x, y) + fmin(x, y) + epsilon)},
 *      onde {@code fmax, fmin} sao os valores de maximo e minimo da janela centrada em (x, y), e
 *      {@code epsilon} e um valor muito pequeno para o caso de o maximo ser zero.</li>
 * <li>{@code pixel = round(L * (1 - D(x, y)))}, onde {@code L} e o valor da maxima
 *      intensidade (255 para 8 bits), e {@code round} e a funcao de arredondamento.</li>
 * </ol>
 * 
 * Note que o {@code D(x, y)} e um valor no intervalo [0, 1], e que quando ele
 * e 1, o pixel e considerado como objeto. Logo seu valor precisa ser invertido,
 * para que o objeto seja o 0, e precisa estar no intervalo [0, L] inteiro.</p>
 * 
 * <p>Os pixels de alto contraste sao obtidos atraves da binarizacao da imagem
 * de contraste, sendo os pixels de alto contraste os que representam os objetos
 * na imagem de contraste. A binarizacao e obtida atraves do metodo de Otsu,
 * implementado em {@link OtsuMethod}.</p>
 * 
 * <p>A parte final, que binariza a imagem de entrada, e um metodo local adaptativo,
 * que obtem um limiar para um certo pixel atraves da media e do desvio padrao
 * levemente alterados da janela centrada neste pixel. Tal media e desvio padrao
 * so levam em conta os pixels da imagem original que correspondem a um pixel de
 * alto contraste. Para que um pixel seja considerado objeto, seu valor nao pode
 * ultrapassar o limiar e a janela em torno dele deve possuir mais do que um
 * numero minimo de pixels de contraste.</p>
 *
 * @author denis
 * @see LuTanMethod
 * @see OtsuMethod
 */
public class SuMethod implements IBinarizationMethod {

	private static final int RECOMMENDED_CONTRAST_WSIZE = 3;
	private final ImageSize windowSize;
	private final int minHighContrastPixels;
	private final ImageSize contrastWindowSize;

	/**
	 * Metodo de Su para o tamanho de janela de binarizacao, numero minimo de
	 * pixels de alto contraste na janela e tamanho de janela para obtencao da
	 * imagem de contraste dados.
	 * 
	 * @param windowSize o tamanho da janela da etapa de binarizacao local adaptativa
	 * @param minHighContrastPixels o numero minimo de pixels de alto contraste na janela
	 * @param contrastWindowSize o tamanho da janela para obtencao da imagem de contraste
	 * @throws IllegalArgumentException caso o tamanho das janelas seja nao-positivo
	 */
	public SuMethod(int windowSize, int minHighContrastPixels, int contrastWindowSize) {
		this.windowSize = ImageSize.of(windowSize);
		this.minHighContrastPixels = minHighContrastPixels;
		this.contrastWindowSize = ImageSize.of(contrastWindowSize);
	}

	/**
	 * Metodo de Su para o tamanho de janela de binarizacao e numero minimo de
	 * pixels de alto contraste na janela dados. Utiliza o tamanho recomendado
	 * de janela para obtencao da imagem de contraste.
	 * 
	 * @param windowSize o tamanho da janela da etapa de binarizacao local adaptativa
	 * @param minHighContrastPixels o numero minimo de pixels de alto contraste na janela
	 */
	public SuMethod(int windowSize, int minHighContrastPixels) {
		this(windowSize, minHighContrastPixels, RECOMMENDED_CONTRAST_WSIZE);
	}

	@Override
	public IBinaryImage binarize(IGrayLevelImage input) {
		final IBinaryImage highContrast = getHighContrastPixels(input);
		final MaskedImage masked = MaskedImage.of(input, highContrast);

		final IBinaryImage.Builder builder = Images.newBinaryImageBuilder(masked.size());

		for (int x = 0; x < masked.size().width(); x++) {
			for (int y = 0; y < masked.size().height(); y++) {
				final ImagePoint p = ImagePoint.of(x, y);

				final MaskedImage window = masked.windowOf(p, windowSize);

				if (window.getNumberOfMasked() >= minHighContrastPixels) {
					final double threshold = window.getMean() + window.getStandardDeviation() / 2.0;

					if (input.pixelAt(p).value() <= threshold) {
						builder.setPixel(p);
					}
				}
			}
		}

		return builder.build();
	}

	/**
	 * Obtem a imagem com os pixels de alto contraste a partir da imagem em niveis
	 * de cinza dada.
	 * 
	 * @param input a imagem em niveis de cinza da qual se quer os pixels de alto contraste
	 * @return a imagem com os pixels de alto contraste da imagem dada
	 */
	private IBinaryImage getHighContrastPixels(IGrayLevelImage input) {
		final IGrayLevelImage.Builder builder = Images.newGrayLevelImageBuilder(input.size());

		for (int x = 0; x < input.size().width(); x++) {
			for (int y = 0; y < input.size().height(); y++) {
				final ImagePoint p = ImagePoint.of(x, y);

				builder.setPixel(p, ContrastType.MAX_MIN.contrastPixelAt(input, contrastWindowSize, p));
			}
		}

		return OtsuMethod.INSTANCE.binarize(builder.build());
	}

	@Override
	public String getLabel() {
		return "su_window" + windowSize.width() + "_nmin" + minHighContrastPixels + "_cwindow" + contrastWindowSize.width();
	}

	@Override
	public String toString() {
		return "Su, Window Size: " + windowSize.width() + ", Nmin: " + minHighContrastPixels + ", Constrast WinSize: " + contrastWindowSize.width();
	}
}
