package binarization.metrics;

import binarization.data.IBinaryImage;
import binarization.data.Pixel;
import binarization.data.ImagePoint;

/**
 * Classe simples acumula os valores de true positive, true negative,
 * false positive e false negative para duas imagens binarias.
 * 
 * <p>Imutavel, e nao pode ser herdada fora do pacote para garantir sua
 * imutabilidade.</p>
 *
 * @author denis
 */
class BinaryImageAccumulator {

	private final long truePositives;
	private final long trueNegatives;
	private final long falsePositives;
	private final long falseNegatives;

	/**
	 * Acumula os valores de true positives, true negatives, false positives e
	 * false negatives entre as imagens dadas, e constroi um objeto com estes
	 * valores.
	 *
	 * @param groundTruth a imagem de comparacao
	 * @param binarized a imagem que vai ser avaliada
	 * @throws IllegalArgumentException se as imagens tiverem tamanhos diferentes
	 */
	BinaryImageAccumulator(IBinaryImage groundTruth, IBinaryImage binarized) {

		if (!groundTruth.size().equals(binarized.size())) {
			throw new IllegalArgumentException("Images must be of equal size.");
		}

		long tp = 0, tn = 0, fp = 0, fn = 0;

		for (int x = 0; x < groundTruth.size().width(); x++) {
			for (int y = 0; y < groundTruth.size().height(); y++) {

				final Pixel pixelGT = groundTruth.pixelAtUnchecked(x, y);
				final Pixel pixelBin = binarized.pixelAtUnchecked(x, y);

				if (pixelGT.isBlack()) {
					if (pixelBin.isBlack()) {
						tp++;
					} else {
						fn++;
					}
				} else {
					if (pixelBin.isBlack()) {
						fp++;
					} else {
						tn++;
					}
				}

			}
		}

		truePositives = tp;
		trueNegatives = tn;
		falsePositives = fp;
		falseNegatives = fn;
	}

	public long getTruePositives() {
		return truePositives;
	}

	public long getTrueNegatives() {
		return trueNegatives;
	}

	public long getFalsePositives() {
		return falsePositives;
	}

	public long getFalseNegatives() {
		return falseNegatives;
	}
}
