package binarization.metrics;

import binarization.IBinaryImageMetric;
import binarization.data.IBinaryImage;
import binarization.data.ImagePoint;
import binarization.data.ImageSize;
import binarization.data.Pixel;
import binarization.util.Comparators;
import java.text.Format;
import java.text.NumberFormat;
import java.util.Comparator;

/**
 *
 * @author denis
 */
public class DistanceReciprocalDistortionMetric implements IBinaryImageMetric {

	public static final DistanceReciprocalDistortionMetric DEFAULT = new DistanceReciprocalDistortionMetric(5, 8);
	private final double[][] weights;
	private final int nonUniformBlockSize;

	public DistanceReciprocalDistortionMetric(int weightedMatrixDimension, int nonUniformBlockSize) {
		if (weightedMatrixDimension % 2 == 0) {
			throw new IllegalArgumentException("Weighted Matrix dimension must be odd.");
		}

		this.weights = calculateWeightedMatrix(weightedMatrixDimension);
		this.nonUniformBlockSize = nonUniformBlockSize;
	}

	private double[][] calculateWeightedMatrix(int dimension) {
		final double[][] result = new double[dimension][dimension];
		final int center = dimension / 2;

		double sum = 0.0;
		for (int i = 0; i < dimension; i++) {
			for (int j = 0; j < dimension; j++) {

				if (i != center || j != center) {
					final int iDiff = i - center;
					final int jDiff = j - center;
					final double value = 1.0 / Math.sqrt(iDiff * iDiff + jDiff * jDiff);

					result[i][j] = value;
					sum += value;

				} else {
					result[i][j] = 0.0;
				}
			}
		}

		//normalization
		for (int i = 0; i < dimension; i++) {
			for (int j = 0; j < dimension; j++) {
				result[i][j] = result[i][j] / sum;
			}
		}

		return result;
	}

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized) {
		final double numerator = calculateTotalDistortion(groundTruth, binarized)
				* Math.abs(Pixel.ofWhite().value() - Pixel.ofBlack().value());

//		System.out.println("numerator: " + numerator + ", nubn: " + countNonUniformBlockNumber(groundTruth));

		return numerator / countNonUniformBlockNumber(groundTruth);
	}

	private double calculateTotalDistortion(IBinaryImage groundTruth, IBinaryImage binarized) {
		final ImageSize windowSize = ImageSize.of(weights.length);
		double result = 0.0;

		for (int x = 0; x < groundTruth.size().width(); x++) {
			for (int y = 0; y < groundTruth.size().height(); y++) {

				final Pixel pixelGT = groundTruth.pixelAtUnchecked(x, y);
				final Pixel pixelBin = binarized.pixelAtUnchecked(x, y);

				if (!pixelGT.equals(pixelBin)) {
					final ImagePoint center = ImagePoint.of(x, y);

					result += distortionOf(
							calculateOffset(center),
							groundTruth.windowOf(center, windowSize),
							pixelBin);
				}
			}
		}

		return result;
	}

	private ImagePoint calculateOffset(ImagePoint center) {
		final int halfWindow = weights.length / 2;
		final int xOffset = halfWindow - center.x();
		final int yOffset = halfWindow - center.y();

		return ImagePoint.of(
				xOffset > 0 ? xOffset : 0,
				yOffset > 0 ? yOffset : 0);
	}

	private double distortionOf(ImagePoint offset, IBinaryImage windowGT, Pixel pixelBin) {
		double result = 0.0;

		for (int x = 0; x < windowGT.size().width(); x++) {
			for (int y = 0; y < windowGT.size().height(); y++) {
				final Pixel pixelGT = windowGT.pixelAtUnchecked(x, y);

				if (!pixelGT.equals(pixelBin)) {
					result += weights[x + offset.x()][y + offset.y()];
				}
			}
		}

		return result;
	}

	private long countNonUniformBlockNumber(IBinaryImage groundTruth) {
		long result = 0;

		for (int x = 0; x < groundTruth.size().width() - nonUniformBlockSize; x++) {
			for (int y = 0; y < groundTruth.size().height() - nonUniformBlockSize; y++) {

				if (!isUniform(groundTruth, ImagePoint.of(x, y))) {
					result++;
				}

			}
		}

		return result;
	}

	private boolean isUniform(IBinaryImage groundTruth, ImagePoint offset) {
		final Pixel first = groundTruth.pixelAt(offset);

		for (int x = 0; x < nonUniformBlockSize; x++) {
			for (int y = 0; y < nonUniformBlockSize; y++) {
				final Pixel current = groundTruth.pixelAtUnchecked(x + offset.x(), y + offset.y());

				if (!first.equals(current)) {
					return false;
				}
			}
		}

		return true;
	}

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized, IBinaryImage skeletonizedGT) {
		//nao utiliza a terceira imagem
		return evaluate(groundTruth, binarized);
	}

	@Override
	public String getLabel() {
		return "DRD";
	}

	@Override
	public Format getFormat() {
		//NumberFormat e mutavel, portanto necessita de copia defensiva
		final NumberFormat format = NumberFormat.getInstance();
		format.setMinimumFractionDigits(2);

		return format;
	}

	@Override
	public Comparator<Double> getResultComparator() {
		return Comparators.DECREASING;
	}

	@Override
	public String toString() {
		return "Distance-Reciprocal Distortion Metric";
	}
}
