package binarization.metrics;

import binarization.IBinaryImageMetric;
import binarization.data.IBinaryImage;
import java.text.Format;
import java.util.Comparator;

/**
 * F-Measure, obtida do Precision e Recall.
 * 
 * <p>Esta metrica avalia a imagem binarizada da seguinte forma: <ol>
 * 
 * <li>Calcula o valor de {@code recall = tp / (tp + fn)} </li>
 * <li>Calcula o valor de {@code precision = tp / (tp + fp)} </li>
 * <li>O valor final e {@code F-Measure = 2*recall*precision / (recall + precision)} </li>
 * </ol></p>
 * 
 * <p>tp, fn e fp sao o numero de true positives, false negatives e false positives,
 * respectivamente.</p>
 * 
 * <p>O valor devolvido esta no intervalo [0, 1], e quanto mais alto for,
 * mais parecida com o ground truth sera a imagem binarizada.</p>
 * 
 * <p><em>Nota da implementacao: e um enum para garantir o padrao singleton, nao e
 * {@code static} para utilizar polimorfismo, e nao sao todos um unico enum
 * para poder ser extensivel e mais organizado.</em></p>
 *
 * @author denis
 * @see PseudoFMeasure
 */
public enum FMeasure implements IBinaryImageMetric {

	/**Unica instancia da classe*/
	INSTANCE;

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized) {
		final BinaryImageAccumulator acc = new BinaryImageAccumulator(groundTruth, binarized);

		final long tp = acc.getTruePositives();
		final long fp = acc.getFalsePositives();
		final long fn = acc.getFalseNegatives();

		final double recall = FMeasureHelper.calculateRecall(tp, fn);
		final double precision = FMeasureHelper.calculatePrecision(tp, fp);

		return FMeasureHelper.calculateFMeasure(recall, precision);
	}

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized, IBinaryImage skeletonizedGT) {
		//nao utiliza a terceira imagem
		return evaluate(groundTruth, binarized);
	}

	@Override
	public String getLabel() {
		return "FM";
	}

	@Override
	public Format getFormat() {
		return FMeasureHelper.getFormat();
	}

	@Override
	public Comparator<Double> getResultComparator() {
		return FMeasureHelper.getResultComparator();
	}

	@Override
	public String toString() {
		return "F-Measure";
	}
}