package binarization.metrics;

import binarization.util.Comparators;
import java.text.Format;
import java.text.NumberFormat;
import java.util.Comparator;

/**
 * Implementa os calculos do F-Measure e Pseudo F-Measure.
 *
 * @author denis
 */
class FMeasureHelper {

	/**
	 * Garante a nao-instanciabilidade da classe.
	 */
	private FMeasureHelper() {
		throw new AssertionError("Static method class shouldn't be instantiated.");
	}

	/**
	 * Devolve o valor de {@code recall = tp / (tp + fn)}.
	 * 
	 * @param tp o numero de true positives
	 * @param fn o numero de false negatives
	 * @return o valor de recall
	 */
	static double calculateRecall(long tp, long fn) {
		final double denominator = tp + fn + Double.MIN_VALUE;
		return tp / denominator;
	}

	/**
	 * Devolve o valor de {@code precision = tp / (tp + fp)}.
	 * 
	 * @param tp o numero de true positives
	 * @param fp o numero de false positives
	 * @return o valor de precision
	 */
	static double calculatePrecision(long tp, long fp) {
		final double denominator = tp + fp + Double.MIN_VALUE;
		return tp / denominator;
	}

	/**
	 * Devolve {@code F-Measure = 2*recall*precision / (recall + precision)}.
	 *
	 * @param recall o valor de recall
	 * @param precision o valor de precision
	 * @return o valor da metrica F-Measure
	 */
	static double calculateFMeasure(double recall, double precision) {
		final double numerator = 2.0 * recall * precision;
		final double denominator = recall + precision + Double.MIN_VALUE;
		return numerator / denominator;
	}

	/**
	 * Devolve o formatador do FMeasure, cujo valor e respresantado por uma
	 * porcentagem com duas casas decimais
	 * 
	 * @return o formatador do FMeasure
	 */
	static Format getFormat() {
		//NumberFormat e mutavel, portanto necessita de copia defensiva
		final NumberFormat format = NumberFormat.getPercentInstance();
		format.setMinimumFractionDigits(2);

		return format;
	}

	/**
	 * Devolve o comparador entre valores do FMeasure.
	 * 
	 * @return o comparador entre valores do FMeasure
	 */
	static Comparator<Double> getResultComparator() {
		return Comparators.INCREASING;
	}
}
