package binarization.metrics;

import binarization.IBinaryImageMetric;
import binarization.data.IBinaryImage;
import binarization.util.Comparators;
import java.text.Format;
import java.text.NumberFormat;
import java.util.Comparator;

/**
 * Negative Rate Metric, ou NRM, mede a taxa de erros da imagem binarizada,
 * 
 * <p>Esta metrica avalia a imagem binarizada da seguinte forma: <ol>
 * 
 * <li>Calcula o valor da taxa de falso negativos {@code nrfn = fn / (fn + tp)} </li>
 * <li>Calcula o valor da taxa de falso positivos {@code nrfp = fp / (fp + tn)} </li>
 * <li>O valor final e {@code nrm = (nrfn + nrfp) / 2} </li>
 * </ol></p>
 * 
 * <p>tp, tn, fn e fp sao o numero de true positives, true negatives,
 * false negatives e false positives, respectivamente.</p>
 * 
 * <p>Quanto mais baixo for o valor devolvido, mais parecida com o ground truth
 * sera a imagem binarizada, ao contrario do F-Measure e do PSNR.</p>
 * 
 * <p><em>Nota da implementacao: e um enum para garantir o padrao singleton, nao e
 * {@code static} para utilizar polimorfismo, e nao sao todos um unico enum
 * para poder ser extensivel e mais organizado.</em></p>
 *
 * @author denis
 */
public enum NegativeRateMetric implements IBinaryImageMetric {

	/**Unica instancia da classe*/
	INSTANCE;

	@Override
	public final double evaluate(IBinaryImage groundTruth, IBinaryImage binarized) {
		final BinaryImageAccumulator acc = new BinaryImageAccumulator(groundTruth, binarized);

		final long tp = acc.getTruePositives();
		final long tn = acc.getTrueNegatives();
		final long fp = acc.getFalsePositives();
		final long fn = acc.getFalseNegatives();

		final double nrfn = calculateFalseNegativeRate(fn, tp);
		final double nrfp = calculateFalsePositiveRate(fp, tn);

		return (nrfn + nrfp) / 2.0;
	}

	@Override
	public double evaluate(IBinaryImage groundTruth, IBinaryImage binarized, IBinaryImage skeletonizedGT) {
		//nao utiliza a terceira imagem
		return evaluate(groundTruth, binarized);
	}

	/**
	 * Devolve a taxa de false negatives {@code nrfn = fn / (fn + tp)}.
	 *
	 * @param fn o numero de false negatives
	 * @param tp o numero de true positives
	 * @return a taxa de false negatives
	 */
	private double calculateFalseNegativeRate(long fn, long tp) {
		final double denominator = fn + tp + Double.MIN_VALUE;
		return fn / denominator;
	}

	/**
	 * Devolve a taxa de false positives {@code nrfp = fp / (fp + tn)}.
	 *
	 * @param fp o numero de false positives
	 * @param tn o numero de true negatives
	 * @return a taxa de false positives
	 */
	private double calculateFalsePositiveRate(long fp, long tn) {
		final double denominator = fp + tn + Double.MIN_VALUE;
		return fp / denominator;
	}

	@Override
	public String getLabel() {
		return "NRM";
	}

	@Override
	public Format getFormat() {
		//NumberFormat e mutavel, portanto necessita de copia defensiva
		final NumberFormat format = NumberFormat.getInstance();
		format.setMinimumFractionDigits(5);

		return format;
	}

	@Override
	public Comparator<Double> getResultComparator() {
		return Comparators.DECREASING;
	}

	@Override
	public String toString() {
		return "Negative Rate Metric";
	}
}
